; pagetab.s - pagetable definition
; Copyright (C) 1997 Advanced RISC Machines Limited.
; All Rights Reserved

; BlankPageTable defines whether 'pagetab.s' exports an empty table,
; or a completed page table. There is an identical define in
; 'encache.c', and the two should be kept in step.
		        GBLL     BlankPageTable
BlankPageTable     	SETL     {FALSE}

; Generate either a blank page table for the C code to fill in, or
; generate a full page table at assemble-time
	IF BlankPageTable
	; Generate a blank page table with the % directive

	AREA		pagetab, ALIGN=14, DATA, NOINIT
	EXPORT		pagetable

pagetable
	% 16*1024

	ELSE
	; Generate a full page table, using repetitive assembly

; Access Permissions - not shifted into position.
NO_ACCESS  * 0 ; Depending on the 'R' and 'S' bit, 0
SVC_R      * 0 ;  represents one of these access
ALL_R      * 0 ;  permissions (see A.R.M. section 7.8)
SVC_RW     * 1
NO_USR_W   * 2
ALL_ACCESS * 3

; U, C and B bits in their correct positions (see A.R.M. section 7.6.5)
U_BIT      * 16
C_BIT      * 8
B_BIT      * 4

; Entry type (see A.R.M. section 7.6.5)
SECTION    * 2
PAGE       * 1
INVALID    * 0

	MACRO
	L1Entry $type, $addr, $dom, $ucb, $acc
	[ $type = SECTION
	  DCD ( (($addr) :AND: &FFF00000) :OR: \
		(($acc) :SHL: 10) :OR: \
		(($dom) :SHL: 5) :OR: $ucb :OR: $type )
	  MEXIT
	]
	[ $type = PAGE
	  DCD ( (($addr) :AND: &FFFFFC00) :OR: \
		(($dom) :SHL: 5) :OR: \
		(($ucb) :AND: U_BIT) :OR: $type )
	|
	  DCD 0 ; Invalid Level 1 Page Table Entry
	]
	MEND

	AREA        pagetab, ALIGN=14, DATA

; Create the pagetable using repetitive assembly. Creates just a table
; of "section" entries. True "page tables" would need to be created in a
; similar fashion.

	EXPORT      pagetable
pagetable
	GBLA    counter

counter SETA 0
	; area 1 - 0->16MB, cacheable
	WHILE counter < 16
	L1Entry SECTION, (counter:SHL:20), 0, \
		U_BIT+C_BIT+B_BIT, ALL_ACCESS
counter SETA counter + 1
	WEND

	; area 2 - up to 2GB, not-cacheable
	WHILE counter < 2047
	L1Entry SECTION, (counter:SHL:20), 0, \
		0, ALL_ACCESS
counter SETA counter + 1
	WEND

	; area 3 - last page under 2GB, cacheable
	L1Entry SECTION, (2048 :SHL: 20), 0, \
		U_BIT+C_BIT+B_BIT, ALL_ACCESS
counter SETA counter + 1

	; area 4 - rest of memory, not cacheable
	WHILE counter < 4096
	L1Entry SECTION, (counter:SHL:20), 0, \
		0, ALL_ACCESS
counter SETA counter + 1
	WEND

	ENDIF

	END

; end of file pagetab.s

